/* 
    =>>> look at error.h
    
  Apr09: move \n at line end to avoid buffering on MPI systems
*/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h> /* sleep() */
#include "error.h"
#ifdef CONFIG_MPI
#include <mpi.h>
#endif

int vvv=3;            /* verbose mode */
int err_fatal=0,      /* public, to inform calling functions */
    err_sum=0;        /* num of all errors, slowing down program */
/* pthread access is uncritical in error case */
extern int mpi_i;     /* CONFIG_MPI */

#define FF fflush(stdout)  // may not help in MPI-environments

/* this is a general error routine, which limits the output
 * ToDo: reset by external signal, send a SIGSTOP to itself? (allow SIGCONT)
 * call via ERR(no) -> err_pg_dbg(no,__FILE__,__LINE__)
 *  cost more code size (12-24Byte 6*x86asm), but gives more info
 * ERR(-1)      = reset
 * ERR(0)       = dummy
 * ERR(1)       = debug info file+line
 * ERR(1..8999) = soft error
 * ERR(9000..)  = hard error
 ToDo: mpi: concat multiple messages, async send to node0, recv from0
   node0: send to all nodes async
 */
void err_pg(int no, const int line, const char *file){
  if (no==1 && (mpi_i&(mpi_i-1))==0)
    printf(" DBG %s L%d node %3d\n", file, line, mpi_i);
  if (no==-1) {
    if (err_fatal && (mpi_i&(mpi_i-1))==0)
      {err_pg(1,line,file);printf("reset ERRNO\n");FF;}
    errno=0; err_fatal=0; }
  if (no>1){
    printf(" ERROR %4d (see doc/error.html); %d.%d errno=%d %s L%d\n",
           no, mpi_i, err_sum, errno, file, line);FF;
    /* output errno-code, man errno fmtmsg strerror oserror */
    if (errno) perror("PERROR");
    /* avoid gigant error log-files (full disk) */
    err_sum++;		/* count errors */
    if (err_sum== 100) {printf(" sleep 2s on error\n");FF;}
    if (err_sum>= 100) sleep(2);
    if (err_sum>=1000) {printf(" sleep 10m\n");FF;sleep(60*10);}
  }
  if (no>err_fatal) err_fatal=no;
#ifdef CONFIG_MPI
  if (no>=9000) MPI_Abort(MPI_COMM_WORLD,1); /* may hang else */
#else
  if (no>=9000) exit(no);
#endif
  return;
}
